@@ -5,7 +5,7 @@
 
 package XML::Compile::Schema;
 use vars '$VERSION';
-$VERSION = '0.83';
+$VERSION = '0.84';
 
 use base 'XML::Compile';
 
@@ -333,4 +333,10 @@ sub elements()
 }
 
 
+sub printIndex(@)
+{   my $self = shift;
+    $self->namespaces->printIndex(@_);
+}
+
+
 1;
