@@ -7,7 +7,7 @@ use strict;
 
 package XML::Compile::Schema::BuiltInTypes;
 use vars '$VERSION';
-$VERSION = '0.83';
+$VERSION = '0.84';
 
 use base 'Exporter';
 
@@ -93,6 +93,11 @@ $builtin_types{boolean} =
  };
 
 
+$builtin_types{pattern} =
+ { example => '*.exe'
+ };
+
+
 $builtin_types{integer} =
  { parse   => \&bigint
  , check   => sub { $_[0] =~ m/^\s*[-+]?\s*\d[\s\d]*$/ }
@@ -397,12 +402,27 @@ sub _valid_ncname($)
 }
 
 # better checks needed
+#  NCName matches pattern [\i-[:]][\c-[:]]*
+sub _ncname($) {sub { $_[0] !~ m/\:/ }}
+
+my $ids = 0;
 $builtin_types{ID} =
+ { parse   => \&_collapse
+ , check   => \&_ncname
+ , example => 'id_'.$ids++
+ };
+
 $builtin_types{IDREF} =
+ { parse   => \&_collapse
+ , check   => \&_ncname
+ , example => 'id-ref'
+ };
+
+
 $builtin_types{NCName} =
 $builtin_types{ENTITY} =
  { parse   => \&_collapse
- , check   => sub { $_[0] !~ m/\:/ }
+ , check   => \&_ncname
  , example => 'label'
  };
 
