@@ -8,7 +8,7 @@ use strict;
 
 package XML::Compile::Schema::Instance;
 use vars '$VERSION';
-$VERSION = '0.83';
+$VERSION = '0.84';
 
 
 use Log::Report 'xml-compile', syntax => 'SHORT';
@@ -169,6 +169,8 @@ sub _collectTypes($)
              $sg = pack_type $sgns, $sgname;
         }
 
+        my $abstract = $node->getAttribute('abstract') || 'false';
+
         unless($defkinds{$local})
         {   mistake __x"ignoring unknown definition-type {local}", type => $local;
             next;
@@ -180,6 +182,7 @@ sub _collectTypes($)
           , ns   => $ns,  name => $name, prefix => $prefix
           , afd  => $afd, efd  => $efd,  schema => $self
           , ref  => $ref, sg   => $sg
+          , abstract => ($abstract eq 'true' || $abstract eq '1')
           };
         weaken($info->{schema});
 
@@ -210,23 +213,33 @@ sub importLocations($)
 
 sub printIndex(;$)
 {   my $self   = shift;
-    my $fh     = shift || select;
+    my $fh     = @_ % 2 ? shift : select;
+    my %args   = @_;
 
     $fh->print("namespace: ", $self->targetNamespace, "\n");
-    if(defined(my $source = $self->source))
-    {   $fh->print("  source: $source\n");
-    }
     if(defined(my $filename = $self->filename))
-    {   $fh->print("  filename: $filename\n");
+    {   $fh->print(" filename: $filename\n");
+    }
+    elsif(defined(my $source = $self->source))
+    {   $fh->print("   source: $source\n");
     }
 
-    foreach my $kind (@defkinds)
+    my @kinds
+     = ! defined $args{kinds}      ? @defkinds
+     : ref $args{kinds} eq 'ARRAY' ? @{$args{kinds}}
+     :                               $args{kinds};
+
+    my $list_abstract = exists $args{list_abstract} ? $args{list_abstract} : 1;
+
+    foreach my $kind (@kinds)
     {   my $table = $self->{$kind};
         keys %$table or next;
-        $fh->print("  definitions of $kind objects:\n");
-        $fh->print("    ", $_->{name}, "\n")
-            for sort {$a->{name} cmp $b->{name}}
-                  values %$table;
+        $fh->print("  definitions of ${kind}s:\n") if @kinds > 1;
+        foreach (sort {$a->{name} cmp $b->{name}} values %$table)
+        {   next if $_->{abstract} && ! $list_abstract;
+            my $abstract = $_->{abstract} ? ' [abstract]' : '';
+            $fh->print("    $_->{name}$abstract\n");
+        }
     }
 }
 
